import { jsxs, jsx, Fragment } from 'react/jsx-runtime';
import { cx } from '@emotion/css';
import { flip, size, useFloating, autoUpdate } from '@floating-ui/react';
import { useVirtualizer } from '@tanstack/react-virtual';
import { useCombobox } from 'downshift';
import { useState, useMemo, useRef, useId, useCallback, useEffect } from 'react';
import '@grafana/data';
import { useStyles2 } from '../../themes/ThemeContext.js';
import 'micro-memoize';
import '@emotion/react';
import 'tinycolor2';
import '../../utils/skeleton.js';
import { t } from '../../utils/i18n.js';
import { Icon } from '../Icon/Icon.js';
import { Input } from '../Input/Input.js';
import { getComboboxStyles } from './getComboboxStyles.js';

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};
function itemToString(item) {
  var _a, _b;
  return (_b = (_a = item == null ? void 0 : item.label) != null ? _a : item == null ? void 0 : item.value.toString()) != null ? _b : "";
}
function itemFilter(inputValue) {
  const lowerCasedInputValue = inputValue.toLowerCase();
  return (item) => {
    var _a, _b;
    return !inputValue || ((_a = item == null ? void 0 : item.label) == null ? void 0 : _a.toLowerCase().includes(lowerCasedInputValue)) || ((_b = item == null ? void 0 : item.value) == null ? void 0 : _b.toString().toLowerCase().includes(lowerCasedInputValue));
  };
}
function estimateSize() {
  return 45;
}
const MIN_HEIGHT = 400;
const INDEX_WIDTH_CALCULATION = 100;
const WIDTH_MULTIPLIER = 7.3;
const Combobox = (_a) => {
  var _b = _a, {
    options,
    onChange,
    value,
    isClearable = false,
    createCustomValue = false,
    id,
    "aria-labelledby": ariaLabelledBy
  } = _b, restProps = __objRest(_b, [
    "options",
    "onChange",
    "value",
    "isClearable",
    "createCustomValue",
    "id",
    "aria-labelledby"
  ]);
  var _a2;
  const [items, setItems] = useState(options);
  const selectedItemIndex = useMemo(() => {
    if (value === null) {
      return null;
    }
    const index = options.findIndex((option) => option.value === value);
    if (index === -1) {
      return null;
    }
    return index;
  }, [options, value]);
  const selectedItem = useMemo(() => {
    if (selectedItemIndex !== null) {
      return options[selectedItemIndex];
    }
    if (value !== null) {
      return {
        label: value.toString(),
        value
      };
    }
    return null;
  }, [selectedItemIndex, options, value]);
  const inputRef = useRef(null);
  const floatingRef = useRef(null);
  const menuId = `downshift-${useId().replace(/:/g, "--")}-menu`;
  const labelId = `downshift-${useId().replace(/:/g, "--")}-label`;
  const styles = useStyles2(getComboboxStyles);
  const [popoverMaxWidth, setPopoverMaxWidth] = useState(void 0);
  const [popoverWidth, setPopoverWidth] = useState(void 0);
  const virtualizerOptions = {
    count: items.length,
    getScrollElement: () => floatingRef.current,
    estimateSize,
    overscan: 4
  };
  const rowVirtualizer = useVirtualizer(virtualizerOptions);
  const {
    getInputProps,
    getMenuProps,
    getItemProps,
    isOpen,
    highlightedIndex,
    setInputValue,
    openMenu,
    closeMenu,
    selectItem
  } = useCombobox({
    menuId,
    labelId,
    inputId: id,
    items,
    itemToString,
    selectedItem,
    onSelectedItemChange: ({ selectedItem: selectedItem2 }) => {
      onChange(selectedItem2);
    },
    defaultHighlightedIndex: selectedItemIndex != null ? selectedItemIndex : 0,
    scrollIntoView: () => {
    },
    onInputValueChange: ({ inputValue }) => {
      const filteredItems = options.filter(itemFilter(inputValue));
      if (createCustomValue && inputValue && filteredItems.findIndex((opt) => opt.label === inputValue) === -1) {
        const customValueOption = {
          label: inputValue,
          // @ts-ignore Type casting needed to make this work when T is a number
          value: inputValue,
          description: t("combobox.custom-value.create", "Create custom value")
        };
        setItems([...filteredItems, customValueOption]);
        return;
      } else {
        setItems(filteredItems);
      }
    },
    onIsOpenChange: ({ isOpen: isOpen2 }) => {
      if (isOpen2) {
        setItems(options);
        return;
      }
    },
    onHighlightedIndexChange: ({ highlightedIndex: highlightedIndex2, type }) => {
      if (type !== useCombobox.stateChangeTypes.MenuMouseLeave) {
        rowVirtualizer.scrollToIndex(highlightedIndex2);
      }
    }
  });
  const onBlur = useCallback(() => {
    var _a3, _b2;
    setInputValue((_b2 = (_a3 = selectedItem == null ? void 0 : selectedItem.label) != null ? _a3 : value == null ? void 0 : value.toString()) != null ? _b2 : "");
  }, [selectedItem, setInputValue, value]);
  const middleware = [
    flip({
      // see https://floating-ui.com/docs/flip#combining-with-shift
      crossAxis: true,
      boundary: document.body
    }),
    size({
      apply({ availableWidth }) {
        setPopoverMaxWidth(availableWidth);
      }
    })
  ];
  const elements = { reference: inputRef.current, floating: floatingRef.current };
  const { floatingStyles } = useFloating({
    strategy: "fixed",
    open: isOpen,
    placement: "bottom-start",
    middleware,
    elements,
    whileElementsMounted: autoUpdate
  });
  const hasMinHeight = isOpen && rowVirtualizer.getTotalSize() >= MIN_HEIGHT;
  useDynamicWidth(items, rowVirtualizer.range, setPopoverWidth);
  return /* @__PURE__ */ jsxs("div", { children: [
    /* @__PURE__ */ jsx(
      Input,
      __spreadValues(__spreadValues({
        suffix: /* @__PURE__ */ jsxs(Fragment, { children: [
          !!value && value === (selectedItem == null ? void 0 : selectedItem.value) && isClearable && /* @__PURE__ */ jsx(
            Icon,
            {
              name: "times",
              className: styles.clear,
              title: t("combobox.clear.title", "Clear value"),
              tabIndex: 0,
              role: "button",
              onClick: () => {
                selectItem(null);
              },
              onKeyDown: (e) => {
                if (e.key === "Enter" || e.key === " ") {
                  selectItem(null);
                }
              }
            }
          ),
          /* @__PURE__ */ jsx(
            Icon,
            {
              name: isOpen ? "search" : "angle-down",
              onClick: () => {
                if (isOpen) {
                  closeMenu();
                } else {
                  openMenu();
                }
              }
            }
          )
        ] })
      }, restProps), getInputProps({
        ref: inputRef,
        /*  Empty onCall to avoid TS error
         *  See issue here: https://github.com/downshift-js/downshift/issues/718
         *  Downshift repo: https://github.com/downshift-js/downshift/tree/master
         */
        onChange: () => {
        },
        onBlur,
        "aria-labelledby": ariaLabelledBy
        // Label should be handled with the Field component
      }))
    ),
    /* @__PURE__ */ jsx(
      "div",
      __spreadProps(__spreadValues({
        className: cx(styles.menu, hasMinHeight && styles.menuHeight, !isOpen && styles.menuClosed),
        style: __spreadProps(__spreadValues({}, floatingStyles), {
          maxWidth: popoverMaxWidth,
          minWidth: (_a2 = inputRef.current) == null ? void 0 : _a2.offsetWidth,
          width: popoverWidth
        })
      }, getMenuProps({
        ref: floatingRef,
        "aria-labelledby": ariaLabelledBy
      })), {
        children: isOpen && /* @__PURE__ */ jsx("ul", { style: { height: rowVirtualizer.getTotalSize() }, className: styles.menuUlContainer, children: rowVirtualizer.getVirtualItems().map((virtualRow) => {
          return /* @__PURE__ */ jsx(
            "li",
            __spreadProps(__spreadValues({
              "data-index": virtualRow.index,
              className: cx(
                styles.option,
                selectedItem && items[virtualRow.index].value === selectedItem.value && styles.optionSelected,
                highlightedIndex === virtualRow.index && styles.optionFocused
              ),
              style: {
                height: virtualRow.size,
                transform: `translateY(${virtualRow.start}px)`
              }
            }, getItemProps({
              item: items[virtualRow.index],
              index: virtualRow.index
            })), {
              children: /* @__PURE__ */ jsxs("div", { className: styles.optionBody, children: [
                /* @__PURE__ */ jsx("span", { className: styles.optionLabel, children: items[virtualRow.index].label }),
                items[virtualRow.index].description && /* @__PURE__ */ jsx("span", { className: styles.optionDescription, children: items[virtualRow.index].description })
              ] })
            }),
            items[virtualRow.index].value + items[virtualRow.index].label
          );
        }) })
      })
    )
  ] });
};
const useDynamicWidth = (items, range, setPopoverWidth) => {
  useEffect(() => {
    if (range === null) {
      return;
    }
    const startVisibleIndex = range == null ? void 0 : range.startIndex;
    const endVisibleIndex = range == null ? void 0 : range.endIndex;
    if (typeof startVisibleIndex === "undefined" || typeof endVisibleIndex === "undefined") {
      return;
    }
    if (startVisibleIndex === 0 || startVisibleIndex % INDEX_WIDTH_CALCULATION === 0 && startVisibleIndex >= INDEX_WIDTH_CALCULATION) {
      let maxLength = 0;
      const calculationEnd = Math.min(items.length, endVisibleIndex + INDEX_WIDTH_CALCULATION);
      for (let i = startVisibleIndex; i < calculationEnd; i++) {
        maxLength = Math.max(maxLength, items[i].label.length);
      }
      setPopoverWidth(maxLength * WIDTH_MULTIPLIER);
    } else if (endVisibleIndex % INDEX_WIDTH_CALCULATION === 0 && endVisibleIndex >= INDEX_WIDTH_CALCULATION) {
      let maxLength = 0;
      const calculationStart = Math.max(0, startVisibleIndex - INDEX_WIDTH_CALCULATION);
      for (let i = calculationStart; i < endVisibleIndex; i++) {
        maxLength = Math.max(maxLength, items[i].label.length);
      }
      setPopoverWidth(maxLength * WIDTH_MULTIPLIER);
    }
  }, [items, range, setPopoverWidth]);
};

export { Combobox };
//# sourceMappingURL=Combobox.js.map
