import { css } from '@emotion/react';

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
function buttonBackgroundMixin(startColor, endColor, textColor = "#fff", textShadow = "0px 1px 0 rgba(0, 0, 0, 0.1)") {
  return {
    backgroundColor: startColor,
    backgroundImage: `linear-gradient(to bottom, ${startColor}, ${endColor})`,
    backgroundRepeat: "repeat-x",
    color: textColor,
    textShadow,
    borderColor: startColor,
    // in these cases the gradient won't cover the background, so we override
    "&:hover, &:focus, &:active, &.active, &.disabled, &[disabled]": {
      color: textColor,
      backgroundImage: "none",
      backgroundColor: startColor
    }
  };
}
function buttonSizeMixin(paddingY, paddingX, fontSize, borderRadius) {
  return {
    padding: `${paddingY} ${paddingX}`,
    fontSize,
    borderRadius
  };
}
function widthMixin(theme, max) {
  let result = {};
  for (let i = 1; i <= max; i++) {
    const width = `${theme.spacing(2 * i)} !important`;
    result[`.width-${i}`] = {
      width
    };
    result[`.max-width-${i}`] = {
      maxWidth: width,
      flexGrow: 1
    };
    result[`.min-width-${i}`] = {
      minWidth: width
    };
    result[`.offset-width-${i}`] = {
      marginLeft: width
    };
  }
  return result;
}
function getUtilityClassStyles(theme) {
  return css(__spreadProps(__spreadValues({
    ".highlight-word": {
      color: theme.v1.palette.orange
    },
    ".hide": {
      display: "none"
    },
    ".show": {
      display: "block"
    },
    ".invisible": {
      // can't avoid type assertion here due to !important
      // eslint-disable-next-line @typescript-eslint/consistent-type-assertions
      visibility: "hidden !important"
    },
    ".absolute": {
      position: "absolute"
    },
    ".flex-grow-1": {
      flexGrow: 1
    },
    ".flex-shrink-1": {
      flexShrink: 1
    },
    ".flex-shrink-0": {
      flexShrink: 0
    },
    ".center-vh": {
      height: "100%",
      display: "flex",
      flexDirection: "column",
      alignItems: "center",
      justifyContent: "center",
      justifyItems: "center"
    },
    ".btn": __spreadProps(__spreadValues({
      display: "inline-flex",
      alignItems: "center",
      justifyContent: "center",
      fontWeight: theme.typography.fontWeightMedium,
      lineHeight: theme.typography.body.lineHeight,
      textAlign: "center",
      verticalAlign: "middle",
      cursor: "pointer",
      border: "none",
      height: `${theme.spacing.gridSize * theme.components.height.md}px`
    }, buttonSizeMixin(
      theme.spacing(0),
      theme.spacing(2),
      `${theme.typography.fontSize}px`,
      theme.shape.radius.default
    )), {
      "&, &:active, &.active": {
        "&:focus, &.focus": {
          outline: "none"
        }
      },
      "&:focus, &:hover": {
        textDecoration: "none"
      },
      "&.focus": {
        textDecoration: "none"
      },
      "&:active, &.active": {
        backgroundImage: "none",
        outline: 0
      },
      "&.disabled, &[disabled], &:disabled": {
        cursor: "not-allowed",
        opacity: 0.65,
        boxShadow: "none",
        pointerEvents: "none"
      }
    }),
    ".btn-small": __spreadProps(__spreadValues({}, buttonSizeMixin(theme.spacing(0.5), theme.spacing(1), theme.typography.size.sm, theme.shape.radius.default)), {
      height: `${theme.spacing.gridSize * theme.components.height.sm}px`
    }),
    // Deprecated, only used by old plugins
    ".btn-mini": __spreadProps(__spreadValues({}, buttonSizeMixin(theme.spacing(0.5), theme.spacing(1), theme.typography.size.sm, theme.shape.radius.default)), {
      height: `${theme.spacing.gridSize * theme.components.height.sm}px`
    }),
    ".btn-success, .btn-primary": __spreadValues({}, buttonBackgroundMixin(theme.colors.primary.main, theme.colors.primary.shade)),
    ".btn-danger": __spreadValues({}, buttonBackgroundMixin(theme.colors.error.main, theme.colors.error.shade)),
    ".btn-secondary": __spreadValues({}, buttonBackgroundMixin(theme.colors.secondary.main, theme.colors.secondary.shade, theme.colors.text.primary)),
    ".btn-inverse": __spreadProps(__spreadValues({}, buttonBackgroundMixin(
      theme.isDark ? theme.v1.palette.dark6 : theme.v1.palette.gray5,
      theme.isDark ? theme.v1.palette.dark5 : theme.v1.palette.gray4,
      theme.colors.text.primary
    )), {
      "&": {
        boxShadow: "none"
      }
    }),
    ".typeahead": {
      zIndex: theme.zIndex.typeahead
    }
  }, widthMixin(theme, 30)), {
    ".row": {
      display: "flex",
      flexWrap: "wrap",
      marginLeft: `calc(${theme.spacing(4)} / -2)`,
      marginRight: `calc(${theme.spacing(4)} / -2)`
    },
    ".container": {
      marginLeft: "auto",
      marginRight: "auto",
      paddingLeft: `calc(${theme.spacing(4)} / 2)`,
      paddingRight: `calc(${theme.spacing(4)} / 2)`,
      [theme.breakpoints.up("sm")]: {
        maxWidth: theme.breakpoints.values.sm
      },
      [theme.breakpoints.up("md")]: {
        maxWidth: theme.breakpoints.values.md
      },
      [theme.breakpoints.up("lg")]: {
        maxWidth: theme.breakpoints.values.lg
      },
      [theme.breakpoints.up("xl")]: {
        maxWidth: theme.breakpoints.values.xl
      }
    }
  }));
}

export { getUtilityClassStyles };
//# sourceMappingURL=utilityClasses.js.map
